package it.unibs.arnaldo.rovineperdute;

import javax.xml.stream.*;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Parser {

    // rovina = città
    public static void parseRovinePerduteXML(String fileName) {

        XMLInputFactory xmlif;
        XMLStreamReader xmlr = null;
        String currentElement = null;

        // momentanea
        List<Integer> link = new ArrayList<>();

        // mappa id -> rovina
        HashMap<Integer, RovinaPerduta> rovineMap = new HashMap<>();

        // mappa id -> link collegamenti
        HashMap<Integer, List<Integer>> mappaCollegamenti = new HashMap<>();

        // variabili momentanee
        int cityId = -1;
        int x = -1;
        int y = -1;
        int h = -1;
        String nome = null;

        try (FileInputStream reader = new FileInputStream(fileName)) {
            xmlif = XMLInputFactory.newInstance();
            xmlr = xmlif.createXMLStreamReader(fileName, reader);

            while (xmlr.hasNext()) {
                int event = xmlr.getEventType();

                switch (event) {
                    case XMLStreamConstants.START_ELEMENT:
                        currentElement = xmlr.getLocalName();
                        if (currentElement.equals("city")) {
                            cityId = Integer.parseInt(xmlr.getAttributeValue(null, "id"));
                            nome = xmlr.getAttributeValue(null, "name");
                            x = Integer.parseInt(xmlr.getAttributeValue(null, "x"));
                            y = Integer.parseInt(xmlr.getAttributeValue(null, "y"));
                            h = Integer.parseInt(xmlr.getAttributeValue(null, "h"));
                        }
                        if (currentElement.equals("link")) {
                            link.add(Integer.parseInt(xmlr.getAttributeValue(null, "to")));
                        }
                        break;

                    case XMLStreamConstants.END_ELEMENT:
                        String endTag = xmlr.getLocalName();

                        if (endTag.equals("city")) {
                            mappaCollegamenti.put(cityId, link);
                            link = new ArrayList<>();
                            RovinaPerduta rovinaPerduta = new RovinaPerduta(new Coordinata(x, y, h), nome);
                            nome = null;
                            x = -1;
                            y = -1;
                            h = -1;

                            if (cityId != -1){
                                rovineMap.put(cityId, rovinaPerduta);
                            }

                            cityId = -1;
                        }

                        break;
                }

                xmlr.next();
            }

            xmlr.close();
            Graph grafo = new Graph();

            for (Integer idx : mappaCollegamenti.keySet()){
                List<Integer> listaIndiciCollegamenti = mappaCollegamenti.get(idx);
                List<RovinaPerduta> listaCollegamenti = new ArrayList<>();
                RovinaPerduta rp = rovineMap.get(idx);

                for(int i : listaIndiciCollegamenti){
                    listaCollegamenti.add(rovineMap.get(i));
                }

                grafo.add(rp, listaCollegamenti);
            }

            System.out.println(grafo);

        } catch (FactoryConfigurationError | XMLStreamException | IOException e) {
            System.out.println("Error reading the file: " + e.getMessage());
        }
    }
}
